from .srem import srem
from .srem import _has_invalid_angle
from .srem import _calc_rayleigh_optical_depth
from .srem import _calc_rayleigh_reflectance
from .srem import _calc_rayleigh_phase
from .srem import _calc_scattering_angle
from .srem import _calc_relative_azimuth_angle
from .srem import _calc_satm
from .srem import _calc_total_transmittance



from typing import Dict
import xml.etree.ElementTree as ET


class MetadataParser(object):
    def __init__(self, metadata_file):
        tree = ET.parse(metadata_file)
        self.root = tree.getroot()

    def get_mean_angles(self, band_id: int) -> Dict[str, float]:
        angles = {}
        for sun_angles in self.root.iter('Mean_Sun_Angle'):
            angles['solar_azimuth_angle_deg'] = \
                float(sun_angles.find('AZIMUTH_ANGLE').text)
            angles['solar_zenith_angle_deg'] = \
                float(sun_angles.find('ZENITH_ANGLE').text)

        for viewing_angles in self.root.iter('Mean_Viewing_Incidence_Angle'):
            attrib = viewing_angles.attrib
            if int(attrib['bandId']) == band_id:
                angles['sensor_azimuth_angle_deg'] = \
                    float(viewing_angles.find('AZIMUTH_ANGLE').text)
                angles['sensor_zenith_angle_deg'] = \
                    float(viewing_angles.find('ZENITH_ANGLE').text)

        return angles

    def get_platform(self) -> str:
        for tile_id in self.root.iter('TILE_ID'):
            platform = tile_id.text[:3]
        return platform



import numpy as np
import pytest

import srem


TEST_WIDTH = 1000
TEST_HEIGHT = 1000


@pytest.fixture
def toa_reflectance():
    return np.random.rand(TEST_HEIGHT, TEST_WIDTH)


@pytest.fixture
def wavelengths():
    # unit is micrometer
    return [0.4430, 0.4820, 0.5615, 0.6545, 0.8650, 1.6085, 2.2005]


@pytest.fixture
def angle_degs_float():
    angle_degs = {
        'solar_azimuth_angle_deg': 158.1,
        'solar_zenith_angle_deg': 53.1,
        'sensor_azimuth_angle_deg': 258.3,
        'sensor_zenith_angle_deg': 16.9
    }
    return angle_degs


@pytest.fixture
def angle_degs_array(angle_degs_float):
    angle_degs = {}
    for k, v in angle_degs_float.items():
        angle_degs[k] = v * np.ones((TEST_HEIGHT, TEST_WIDTH)) + np.random.rand(TEST_HEIGHT, TEST_WIDTH)
    return angle_degs


@pytest.fixture
def invalid_angle_degs_array(angle_degs_float):
    angle_degs = {}
    for k, v in angle_degs_float.items():
        angle_degs[k] = v * np.ones((TEST_HEIGHT + 1, TEST_WIDTH + 1)) + np.random.rand(TEST_HEIGHT + 1, TEST_WIDTH + 1)
    return angle_degs


@pytest.fixture
def angle_rads_float(angle_degs_float):
    angle_rads = {}
    for k, v in angle_degs_float.items():
        key = k.replace('_deg', '')
        angle_rads[key] = np.deg2rad(v)
    return angle_rads


@pytest.fixture
def angle_rads_array(angle_degs_array):
    angle_rads = {}
    for k, v in angle_degs_array.items():
        key = k.replace('_deg', '')
        angle_rads[key] = np.deg2rad(v)
    return angle_rads


@pytest.fixture
def rayleigh_optical_depths(wavelengths):
    return [srem._calc_rayleigh_optical_depth(w) for w in wavelengths]



import pytest

from constants import BAND_ID
from metadata_parser import MetadataParser


def test_get_mean_angles(S2A_metadata_file, angle_keys):
    m_parser = MetadataParser(S2A_metadata_file)
    for band_id in BAND_ID:
        angles = m_parser.get_mean_angles(band_id.value)
        assert set(angles.keys()) == angle_keys
        for angle in angles.values():
            assert isinstance(angle, float)


def test_get_platform(S2A_metadata_file, S2B_metadata_file):
    _test_get_platform(S2A_metadata_file, 'S2A')
    _test_get_platform(S2B_metadata_file, 'S2B')


def _test_get_platform(metadata_file, expected_platform):
    m_parser = MetadataParser(metadata_file)
    platform = m_parser.get_platform()
    assert platform == expected_platform



import glob
import os
from typing import Any, Dict, List, Tuple
import subprocess

import numpy as np
from rasterio.windows import Window
import srem

from constants import OLI_BAND_ID, REFLECTANCE_SCALING_FACTOR


def get_band_id(path: str) -> int:
    band_name = os.path.splitext(os.path.basename(path))[0].split('_')[-1]
    band_id = OLI_BAND_ID[band_name].value
    return band_id


def get_pixel_angle_files(angle_file: str,
                          band_id: int,
                          output_dir: str) -> Tuple[str, str]:
    cwd = os.getcwd()
    angle_file = os.path.abspath(angle_file)
    os.chdir(output_dir)
    cmd = f'l8_angles {angle_file} BOTH 1 -b {band_id}'
    subprocess.run(cmd, shell=True, check=True, stdout=subprocess.DEVNULL)
    solar_angle_file = glob.glob(os.path.join(output_dir, f'*solar_B0{band_id}.img'))[0]
    sensor_angle_file = glob.glob(os.path.join(output_dir, f'*sensor_B0{band_id}.img'))[0]
    os.chdir(cwd)
    return solar_angle_file, sensor_angle_file


def srem_worker(data: List[np.ndarray],
                window: Window,
                ij: int,
                global_args: Dict[Any, Any]) -> np.ndarray:
    nodata_mask = (data[0] == 0)
    surface_reflectance = srem.srem(
        toa_reflectance=data[0],
        wavelength=global_args['wavelength'],
        solar_azimuth_angle_deg=data[1] / 100.,
        solar_zenith_angle_deg=data[2] / 100.,
        sensor_azimuth_angle_deg=data[3] / 100.,
        sensor_zenith_angle_deg=data[4] / 100.,
    )
    # surface reflectance is scaled in this example.
    scaled_sr = \
        surface_reflectance * REFLECTANCE_SCALING_FACTOR
    # crop values less than 1 for defining 1 as minimum value.
    scaled_sr[scaled_sr < 1] = 1
    scaled_sr[nodata_mask] = 0
    scaled_sr = scaled_sr.astype(global_args['dtype'])
    scaled_sr = np.expand_dims(scaled_sr, axis=0)
    return scaled_sr



import argparse
import os
from typing import List

from tqdm import tqdm

from apply_srem import apply_srem


def main(input_raster_files: List[str],
         metadata_file: str,
         angle_file: str,
         output_raster_dir: str) -> None:
    os.makedirs(output_raster_dir, exist_ok=True)
    for input_raster_file in tqdm(input_raster_files):
        output_raster_file = os.path.join(
            output_raster_dir,
            os.path.basename(input_raster_file))
        apply_srem(
            input_raster_file=input_raster_file,
            metadata_file=metadata_file,
            angle_file=angle_file,
            output_raster_file=output_raster_file)


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument('-i', '--input_raster_files', nargs='*', required=True,
                        help='Path to input raster files')
    parser.add_argument('-m', '--metadata_file', required=True,
                        help='Path to metadata file')
    parser.add_argument('-a', '--angle_file', required=True,
                        help='Path to angle file')
    parser.add_argument('-o', '--output_raster_dir', required=True,
                        help='Path to output directory. Each file name is '
                             'same as input raster file name in this example.')
    args = parser.parse_args()

    main(
        input_raster_files=args.input_raster_files,
        metadata_file=args.metadata_file,
        angle_file=args.angle_file,
        output_raster_dir=args.output_raster_dir)



import os

import rasterio
import pytest

from constants import OLI_BAND_ID
import utils


def test_band_id(raster_files):
    for raster_file, expected_band in zip(raster_files, OLI_BAND_ID):
        band_id = utils.get_band_id(raster_file)
        assert band_id == expected_band.value


def test_pixel_angle_files(raster_files, angle_file, tmpdir):
    for raster_file in raster_files:
        with rasterio.open(raster_file) as src:
            expected_height = src.height
            expected_width = src.width
        band_id = utils.get_band_id(raster_file)
        angle_files = utils.get_pixel_angle_files(angle_file, band_id, tmpdir)
        assert len(angle_files) == 2
        for f in angle_files:
            assert os.path.exists(f)
            with rasterio.open(f) as src:
                assert src.height == expected_height
                assert src.width == expected_width


def test_srem_worker(input_srem_worker):
    output = utils.srem_worker(**input_srem_worker)
    assert output.shape == (1, *input_srem_worker['data'].shape[1:])
    assert str(output.dtype) == input_srem_worker['global_args']['dtype']



import argparse
import os
import tempfile

import rasterio
import riomucho
from rio_toa import reflectance

from constants import OLI_WAVELENGTHS
import utils


def apply_srem(input_raster_file: str,
               metadata_file: str,
               angle_file: str,
               output_raster_file: str) -> None:
    out_dir = os.path.join(os.path.dirname(output_raster_file))
    os.makedirs(out_dir, exist_ok=True)
    band_id = utils.get_band_id(input_raster_file)
    wavelength = OLI_WAVELENGTHS[band_id]

    with tempfile.TemporaryDirectory() as tmp_dir:
        toa_file = os.path.join(tmp_dir, 'toa.tif')
        reflectance.calculate_landsat_reflectance(
            [input_raster_file],
            metadata_file,
            toa_file,
            rescale_factor=1.0,
            creation_options={},
            bands=[band_id],
            dst_dtype='float32',
            processes=1,
            pixel_sunangle=True,
            clip=False)
        solar_angle_file, sensor_angle_file = \
            utils.get_pixel_angle_files(angle_file, band_id, tmp_dir)

        with rasterio.open(input_raster_file) as src:
            options = src.profile
            options.update(nodata=0)
        with riomucho.RioMucho(
                [toa_file, solar_angle_file, sensor_angle_file],
                output_raster_file,
                utils.srem_worker,
                mode='array_read',
                global_args={
                    'wavelength': wavelength,
                    'dtype': options['dtype']},
                options=options) as rios:
            rios.run(4)


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument('-i', '--input_raster_file', required=True,
                        help='Path to input raster file')
    parser.add_argument('-m', '--metadata_file', required=True,
                        help='Path to metadata file')
    parser.add_argument('-a', '--angle_file', required=True,
                        help='Path to angle file')
    parser.add_argument('-o', '--output_raster_file', required=True,
                        help='Path to output raster file. GeoTiff format '
                             'is used in this example.')
    args = parser.parse_args()

    apply_srem(
        input_raster_file=args.input_raster_file,
        metadata_file=args.metadata_file,
        angle_file=args.angle_file,
        output_raster_file=args.output_raster_file)



from enum import auto, Enum


class OLI_BAND_ID(Enum):
    B1 = auto()
    B2 = auto()
    B3 = auto()
    B4 = auto()
    B5 = auto()
    B6 = auto()
    B7 = auto()
    B8 = auto()
    B9 = auto()


# unit is micrometer.
# See https://landsat.gsfc.nasa.gov/preliminary-spectral-response-of-the-operational-land-imager-in-band-band-average-relative-spectral-response/
OLI_WAVELENGTHS = {
    1: 0.44296,
    2: 0.48204,
    3: 0.56141,
    4: 0.65459,
    5: 0.86467,
    6: 1.60886,
    7: 2.20073,
    8: 0.58950,
    9: 1.37343
}


REFLECTANCE_SCALING_FACTOR = 10000



import glob
import os
import shutil
import tempfile

import pytest
import rasterio

from main import apply_srem


def test_apply_srem(S2A_raster_files,
                    S2A_metadata_file,
                    S2B_raster_files,
                    S2B_metadata_file):
    _test_apply_srem(S2A_raster_files, S2A_metadata_file)
    _test_apply_srem(S2B_raster_files, S2B_metadata_file)


def _test_apply_srem(input_raster_files, metadata_file):
    for input_raster_file in input_raster_files:
        with tempfile.TemporaryDirectory() as tmp_dir:
            output_raster_file = os.path.join(tmp_dir, 'boa.tif')
            apply_srem(
                input_raster_file=input_raster_file,
                metadata_file=metadata_file,
                output_raster_file=output_raster_file)
            assert os.path.exists(output_raster_file)
            with rasterio.open(input_raster_file) as src_ds, \
                 rasterio.open(output_raster_file) as dst_ds:
                assert src_ds.count == dst_ds.count
                assert src_ds.width == dst_ds.width
                assert src_ds.height == dst_ds.height
                assert src_ds.dtypes == dst_ds.dtypes
                assert src_ds.crs == dst_ds.crs
                assert src_ds.transform == dst_ds.transform



import argparse
import os

import rasterio
import srem

from metadata_parser import MetadataParser
from constants import (
    BAND_ID,
    WAVELENGTHS,
    REFLECTANCE_SCALING_FACTOR
)


def apply_srem(input_raster_file: str,
               metadata_file: str,
               output_raster_file: str) -> None:
    out_dir = os.path.join(os.path.dirname(output_raster_file))
    os.makedirs(out_dir, exist_ok=True)
    m_parser = MetadataParser(metadata_file)
    band_id = BAND_ID[os.path.splitext(input_raster_file)[0][-3:]].value
    platform = m_parser.get_platform()
    wavelength = WAVELENGTHS[platform][band_id]
    angles = m_parser.get_mean_angles(band_id)

    with rasterio.open(input_raster_file) as src:
        toa_reflectance = src.read(1) / REFLECTANCE_SCALING_FACTOR
        profile = src.profile
        nodata_mask = (toa_reflectance == 0)

    surface_reflectance = srem.srem(
        toa_reflectance=toa_reflectance,
        wavelength=wavelength,
        **angles)
    scaled_surface_reflectance = \
        surface_reflectance * REFLECTANCE_SCALING_FACTOR
    # crop values less than 1 for defining 1 as minimum value.
    scaled_surface_reflectance[scaled_surface_reflectance < 1] = 1
    scaled_surface_reflectance[nodata_mask] = 0

    profile.update(
        driver='GTiff',
        compress='deflate',
        nodata=0)
    with rasterio.open(output_raster_file, 'w', **profile) as dst:
        dst.write(
            scaled_surface_reflectance.astype(profile['dtype']),
            indexes=1)


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument('-i', '--input_raster_file', required=True,
                        help='Path to input raster file')
    parser.add_argument('-m', '--metadata_file', required=True,
                        help='Path to metadata file')
    parser.add_argument('-o', '--output_raster_file', required=True,
                        help='Path to output raster file. GeoTiff format '
                             'is used in this example.')
    args = parser.parse_args()

    apply_srem(
        input_raster_file=args.input_raster_file,
        metadata_file=args.metadata_file,
        output_raster_file=args.output_raster_file)



from typing import List, Tuple, Union

import numpy as np


def srem(toa_reflectance: np.ndarray,
         wavelength: float,
         solar_azimuth_angle_deg: Union[float, np.ndarray],
         solar_zenith_angle_deg: Union[float, np.ndarray],
         sensor_azimuth_angle_deg: Union[float, np.ndarray],
         sensor_zenith_angle_deg: Union[float, np.ndarray]) -> np.ndarray:
    """Calculates surface reflectance

    See: https://www.mdpi.com/2072-4292/11/11/1344

    Args:
        toa_reflectance (numpy.ndarray): Top of atmosphere reflectance.
            Expected shape == (height, width)
        wavelength (float): Wavelength of toa_reflectance in micrometer.
        solar_azimuth_angle_deg (float or numpy.ndarray):
            Solar azimuth angle. Expected unit is degree. Height and width have
            to be same as those of toa_reflectance when using numpy.ndarray.
        solar_zenith_angle_deg (float or numpy.ndarray):
            Solar zenith angle. Expected unit is degree. Height and width have
            to be same as those of toa_reflectance when using numpy.ndarray.
        sensor_azimuth_angle_deg (float or numpy.ndarray):
            Sensor azimuth angle. Expected unit is degree. Height and width have
            to be same as those of toa_reflectance when using numpy.ndarray.
        sensor_zenith_angle_deg (float or numpy.ndarray):
            Sensor zenith angle. Expected unit is degree. Height and width have
            to be same as those of toa_reflectance when using numpy.ndarray.

    Returns:
        numpy.ndarray: Surface reflectance with same shape as toa_reflectance
    """
    if _has_invalid_angle(
            angles=[
                solar_azimuth_angle_deg,
                solar_zenith_angle_deg,
                sensor_azimuth_angle_deg,
                sensor_zenith_angle_deg],
            shape=toa_reflectance.shape):
        raise ValueError('An angle have to be float or numpy.ndarray that has same shape as toa_reflectance.')

    solar_azimuth_angle = np.deg2rad(solar_azimuth_angle_deg)
    solar_zenith_angle = np.deg2rad(solar_zenith_angle_deg)
    sensor_azimuth_angle = np.deg2rad(sensor_azimuth_angle_deg)
    sensor_zenith_angle = np.deg2rad(sensor_zenith_angle_deg)

    rayleigh_optical_depth = _calc_rayleigh_optical_depth(wavelength)
    rayleigh_reflectance = _calc_rayleigh_reflectance(
        wavelength, solar_azimuth_angle, solar_zenith_angle,
        sensor_azimuth_angle, sensor_zenith_angle, rayleigh_optical_depth)
    satm = _calc_satm(rayleigh_optical_depth)
    total_transmittance = _calc_total_transmittance(
        solar_zenith_angle, sensor_zenith_angle, rayleigh_optical_depth)
    surface_reflectance = \
        (toa_reflectance - rayleigh_reflectance) \
        / ((toa_reflectance - rayleigh_reflectance) * satm + total_transmittance)

    return surface_reflectance


def _has_invalid_angle(angles: List[Union[float, np.ndarray]],
                       shape: Tuple[int, int]) -> bool:
    for angle in angles:
        if isinstance(angle, np.ndarray) and angle.shape != shape:
            return True
    return False


def _calc_rayleigh_optical_depth(wavelength: float) -> float:
    rayleigh_optical_depth = \
        0.008569 * (1 / (wavelength ** 4)) \
        * (1 + 0.0113 * (1 / (wavelength ** 2)) + 0.00013 * (1 / (wavelength ** 4)))
    return rayleigh_optical_depth


def _calc_rayleigh_reflectance(wavelength: float,
                               solar_azimuth_angle: Union[float, np.ndarray],
                               solar_zenith_angle: Union[float, np.ndarray],
                               sensor_azimuth_angle: Union[float, np.ndarray],
                               sensor_zenith_angle: Union[float, np.ndarray],
                               rayleigh_optical_depth: float) -> Union[float, np.ndarray]:
    rayleigh_phase = _calc_rayleigh_phase(
        solar_azimuth_angle, solar_zenith_angle, sensor_azimuth_angle, sensor_zenith_angle)
    us = np.cos(solar_zenith_angle)
    uv = np.cos(sensor_zenith_angle)
    air_mass = 1 / us + 1 / uv
    rayleigh_reflectance = \
        (rayleigh_phase * (1 - np.exp(-1 * air_mass * rayleigh_optical_depth))) \
        / (4 * (us + uv))
    return rayleigh_reflectance


def _calc_rayleigh_phase(solar_azimuth_angle: Union[float, np.ndarray],
                         solar_zenith_angle: Union[float, np.ndarray],
                         sensor_azimuth_angle: Union[float, np.ndarray],
                         sensor_zenith_angle: Union[float, np.ndarray]) -> Union[float, np.ndarray]:
    coef_a = 0.9587256
    coef_b = 1 - coef_a
    scattering_angle = _calc_scattering_angle(
        solar_azimuth_angle, solar_zenith_angle, sensor_azimuth_angle, sensor_zenith_angle)
    rayleigh_phase = 3 * coef_a * (1 + np.cos(scattering_angle)) / (4 + coef_b)
    return rayleigh_phase


def _calc_scattering_angle(solar_azimuth_angle: Union[float, np.ndarray],
                           solar_zenith_angle: Union[float, np.ndarray],
                           sensor_azimuth_angle: Union[float, np.ndarray],
                           sensor_zenith_angle: Union[float, np.ndarray]) -> Union[float, np.ndarray]:
    relative_azimuth_angle = _calc_relative_azimuth_angle(solar_azimuth_angle, sensor_azimuth_angle)
    scattering_angle = np.arccos(
        -1 * np.cos(sensor_zenith_angle) * np.cos(solar_zenith_angle)
        + np.sin(sensor_zenith_angle) * np.sin(solar_zenith_angle) * np.cos(relative_azimuth_angle))
    return scattering_angle


def _calc_relative_azimuth_angle(angle_1: Union[float, np.ndarray],
                                 angle_2: Union[float, np.ndarray]) -> Union[float, np.ndarray]:
    delta_phi = angle_1 - angle_2
    delta_phi = np.where(delta_phi > 2 * np.pi, delta_phi - 2 * np.pi, delta_phi)
    delta_phi = np.where(delta_phi < 0, delta_phi + 2 * np.pi, delta_phi)
    relative_azimuth_angle = np.abs(delta_phi - np.pi)
    return relative_azimuth_angle


def _calc_satm(rayleigh_optical_depth: float) -> float:
    satm = 0.92 * rayleigh_optical_depth * (np.exp(-1 * rayleigh_optical_depth))
    return satm


def _calc_total_transmittance(solar_zenith_angle: Union[float, np.ndarray],
                              sensor_zenith_angle: Union[float, np.ndarray],
                              rayleigh_optical_depth: float) -> Union[float, np.ndarray]:
    us = np.cos(solar_zenith_angle)
    uv = np.cos(sensor_zenith_angle)
    ts = np.exp(-1 * rayleigh_optical_depth / us) \
        + np.exp(-1 * rayleigh_optical_depth / us) * (np.exp(0.52 * rayleigh_optical_depth / us) - 1)
    tv = np.exp(-1 * rayleigh_optical_depth / uv) \
        + np.exp(-1 * rayleigh_optical_depth / uv) * (np.exp(0.52 * rayleigh_optical_depth / uv) - 1)
    total_transmittance = ts * tv
    return total_transmittance



import numpy as np
import pytest

import srem


def test_srem_with_float_angle(toa_reflectance, wavelengths, angle_degs_float):
    for wavelength in wavelengths:
        surface_reflectance = srem.srem(
            toa_reflectance, wavelength, **angle_degs_float
        )
        assert surface_reflectance.shape == toa_reflectance.shape
        assert surface_reflectance.dtype == toa_reflectance.dtype


def test_srem_with_valid_array_angle(toa_reflectance, wavelengths, angle_degs_array):
    for wavelength in wavelengths:
        surface_reflectance = srem.srem(
            toa_reflectance, wavelength, **angle_degs_array
        )
        assert surface_reflectance.shape == toa_reflectance.shape
        assert surface_reflectance.dtype == toa_reflectance.dtype


def test_srem_with_invalid_array_angle(toa_reflectance, wavelengths, invalid_angle_degs_array):
    with pytest.raises(ValueError):
        _ = srem.srem(
            toa_reflectance, wavelengths[0], **invalid_angle_degs_array
        )

def test_check_valid_angles(toa_reflectance, angle_degs_array):
    assert not srem._has_invalid_angle(angle_degs_array.values(), toa_reflectance.shape)


def test_check_invalid_angles(toa_reflectance, invalid_angle_degs_array):
    assert srem._has_invalid_angle(invalid_angle_degs_array.values(), toa_reflectance.shape)


def test_calc_rayleigh_optical_depth(wavelengths):
    for wavelength in wavelengths:
        rayleigh_optical_depth = srem._calc_rayleigh_optical_depth(wavelength)
        assert isinstance(rayleigh_optical_depth, float)
        assert rayleigh_optical_depth > 0


def test_calc_rayleigh_reflectance_with_float_angle(wavelengths, angle_rads_float, rayleigh_optical_depths):
    for wavelength, rayleigh_optical_depth in zip(wavelengths, rayleigh_optical_depths):
        rayleigh_reflectance = srem._calc_rayleigh_reflectance(
            wavelength=wavelength,
            rayleigh_optical_depth=rayleigh_optical_depth,
            **angle_rads_float)
        assert isinstance(rayleigh_reflectance, float)


def test_calc_rayleigh_reflectance_with_array_angle(wavelengths, angle_rads_array, rayleigh_optical_depths, toa_reflectance):
    for wavelength, rayleigh_optical_depth in zip(wavelengths, rayleigh_optical_depths):
        rayleigh_reflectance = srem._calc_rayleigh_reflectance(
            wavelength=wavelength,
            rayleigh_optical_depth=rayleigh_optical_depth,
            **angle_rads_array)
        assert isinstance(rayleigh_reflectance, np.ndarray)
        assert rayleigh_reflectance.shape == toa_reflectance.shape


def test_calc_rayleigh_phase_with_float_angle(angle_rads_float):
    rayleigh_phase = srem._calc_rayleigh_phase(**angle_rads_float)
    assert isinstance(rayleigh_phase, float)
    assert rayleigh_phase >= 0


def test_calc_rayleigh_phase_with_array_angle(angle_rads_array, toa_reflectance):
    rayleigh_phase = srem._calc_rayleigh_phase(**angle_rads_array)
    assert isinstance(rayleigh_phase, np.ndarray)
    assert rayleigh_phase.shape == toa_reflectance.shape
    assert rayleigh_phase.min() >= 0


def test_calc_scattering_angle_with_float_angle(angle_rads_float):
    scattering_angle = srem._calc_scattering_angle(**angle_rads_float)
    assert isinstance(scattering_angle, float)
    assert 0 <= scattering_angle <= np.pi


def test_calc_scattering_angle_with_array_angle(angle_rads_array, toa_reflectance):
    scattering_angle = srem._calc_rayleigh_phase(**angle_rads_array)
    assert isinstance(scattering_angle, np.ndarray)
    assert scattering_angle.shape == toa_reflectance.shape
    assert scattering_angle.min() >= 0
    assert scattering_angle.max() <= np.pi


def test_calc_relative_azimuth_angle_with_float_angle(angle_rads_float):
    relative_azimuth_angle = srem._calc_relative_azimuth_angle(
        angle_1=angle_rads_float['solar_azimuth_angle'],
        angle_2=angle_rads_float['sensor_azimuth_angle'])
    assert isinstance(relative_azimuth_angle, float)
    assert 0 <= relative_azimuth_angle <= np.pi


def test_calc_relative_azimuth_angle_with_array_angle(angle_rads_array, toa_reflectance):
    relative_azimuth_angle = srem._calc_relative_azimuth_angle(
        angle_1=angle_rads_array['solar_azimuth_angle'],
        angle_2=angle_rads_array['sensor_azimuth_angle'])
    assert isinstance(relative_azimuth_angle, np.ndarray)
    assert relative_azimuth_angle.shape == toa_reflectance.shape
    assert relative_azimuth_angle.min() >= 0
    assert relative_azimuth_angle.max() <= np.pi


def test_calc_satm(rayleigh_optical_depths):
    for rayleigh_optical_depth in rayleigh_optical_depths:
        satm = srem._calc_satm(rayleigh_optical_depth)
        assert isinstance(satm, float)
        assert satm > 0


def test_total_transmittance_with_float_angle(angle_rads_float, rayleigh_optical_depths):
    for rayleigh_optical_depth in rayleigh_optical_depths:
        total_transmittance = srem._calc_total_transmittance(
            solar_zenith_angle=angle_rads_float['solar_zenith_angle'],
            sensor_zenith_angle=angle_rads_float['sensor_zenith_angle'],
            rayleigh_optical_depth=rayleigh_optical_depth
        )
        assert isinstance(total_transmittance, float)
        assert total_transmittance > 0


def test_calc_total_transmittance_with_array_angle(angle_rads_array, rayleigh_optical_depths, toa_reflectance):
    for rayleigh_optical_depth in rayleigh_optical_depths:
        total_transmittance = srem._calc_total_transmittance(
            solar_zenith_angle=angle_rads_array['solar_zenith_angle'],
            sensor_zenith_angle=angle_rads_array['sensor_zenith_angle'],
            rayleigh_optical_depth=rayleigh_optical_depth
        )
        assert isinstance(total_transmittance, np.ndarray)
        assert total_transmittance.shape == toa_reflectance.shape
        assert total_transmittance.min() > 0



import pytest

from constants import BAND_ID, WAVELENGTHS


@pytest.mark.parametrize(
    "platform, expected",
    (
        ['S2A', len(BAND_ID)],
        ['S2B', len(BAND_ID)]
    )
)
def test_num_wavelengths(platform, expected):
    len(WAVELENGTHS[platform]) == expected



import glob
import os
import subprocess

import pytest

TEST_S2A_PRODUCT_URL = 's3://sentinel-s2-l1c/tiles/18/T/WL/2020/3/1/0/'
TEST_S2B_PRODUCT_URL = 's3://sentinel-s2-l1c/tiles/18/T/WL/2020/2/22/0/'


@pytest.fixture(scope='session')
def S2A_raster_files(tmpdir_factory):
    dst_dir = tmpdir_factory.mktemp('S2A')
    cmd = (f'aws s3 sync {TEST_S2A_PRODUCT_URL} {str(dst_dir)} '
            '--request-payer --exclude "*" --include "B*.jp2"')
    subprocess.run(cmd, shell=True, check=True, stdout=subprocess.DEVNULL)
    file_paths = glob.glob(os.path.join(dst_dir, 'B*.jp2'))
    return file_paths


@pytest.fixture(scope='session')
def S2B_raster_files(tmpdir_factory):
    dst_dir = tmpdir_factory.mktemp('S2B')
    cmd = (f'aws s3 sync {TEST_S2B_PRODUCT_URL} {str(dst_dir)} '
            '--request-payer --exclude "*" --include "B*.jp2"')
    subprocess.run(cmd, shell=True, check=True, stdout=subprocess.DEVNULL)
    file_paths = glob.glob(os.path.join(dst_dir, 'B*.jp2'))
    return file_paths


@pytest.fixture(scope='session')
def S2A_metadata_file(tmpdir_factory):
    dst_dir = tmpdir_factory.mktemp('S2A')
    cmd = (f'aws s3 sync {TEST_S2A_PRODUCT_URL} {str(dst_dir)} '
            '--request-payer --exclude "*" --include "metadata.xml"')
    subprocess.run(cmd, shell=True, check=True, stdout=subprocess.DEVNULL)
    file_path = os.path.join(dst_dir, 'metadata.xml')
    return file_path


@pytest.fixture(scope='session')
def S2B_metadata_file(tmpdir_factory):
    dst_dir = tmpdir_factory.mktemp('S2B')
    cmd = (f'aws s3 sync {TEST_S2B_PRODUCT_URL} {str(dst_dir)} '
            '--request-payer --exclude "*" --include "metadata.xml"')
    subprocess.run(cmd, shell=True, check=True, stdout=subprocess.DEVNULL)
    file_path = os.path.join(dst_dir, 'metadata.xml')
    return file_path


@pytest.fixture
def angle_keys():
    keys = set([
        'sensor_azimuth_angle_deg',
        'sensor_zenith_angle_deg',
        'solar_azimuth_angle_deg',
        'solar_zenith_angle_deg'])
    return keys



#!/usr/bin/env python

import setuptools


readme = open('README.md').read()
long_description = '\n'.join([str(line) for line in readme.split('\n')[3:]])

setuptools.setup(
    long_description=long_description,
    long_description_content_type='text/markdown',
    use_scm_version=True,
    setup_requires=['setuptools_scm'],
)



import glob
import os
import subprocess

import numpy as np
import pytest
import rasterio
from rasterio.windows import Window

from constants import OLI_WAVELENGTHS
import utils


TEST_ENDPOINT = 's3://landsat-pds/c1/L8/013/032/LC08_L1TP_013032_20200222_20200225_01_T1/'
TEST_WINDOW = Window(256, 256, 256, 256)


@pytest.fixture(scope='session')
def raster_files(tmpdir_factory):
    tmpdir = tmpdir_factory.mktemp('data')
    cmd = (f'aws s3 sync {TEST_ENDPOINT} {str(tmpdir)} '
            '--exclude "*" --include "*B[1-9].TIF"')
    subprocess.run(cmd, shell=True, check=True, stdout=subprocess.DEVNULL)
    file_paths = sorted(glob.glob(os.path.join(tmpdir, '*B[1-9].TIF')))
    return file_paths


@pytest.fixture(scope='session')
def metadata_file(tmpdir_factory):
    tmpdir = tmpdir_factory.mktemp('data')
    cmd = (f'aws s3 sync {TEST_ENDPOINT} {str(tmpdir)} '
            '--exclude "*" --include "*MTL.json"')
    subprocess.run(cmd, shell=True, check=True, stdout=subprocess.DEVNULL)
    file_path = glob.glob(os.path.join(tmpdir, '*MTL.json'))[0]
    return file_path


@pytest.fixture(scope='session')
def angle_file(tmpdir_factory):
    tmpdir = tmpdir_factory.mktemp('data')
    cmd = (f'aws s3 sync {TEST_ENDPOINT} {str(tmpdir)} '
            '--exclude "*" --include "*ANG.txt"')
    subprocess.run(cmd, shell=True, check=True, stdout=subprocess.DEVNULL)
    file_path = glob.glob(os.path.join(tmpdir, '*ANG.txt'))[0]
    return file_path


@pytest.fixture
def cropped_data(raster_files):
    with rasterio.open(raster_files[0]) as src:
        return src.read(window=TEST_WINDOW)


@pytest.fixture
def cropped_angle(raster_files, angle_file, tmpdir):
    raster_file = raster_files[0]
    band_id = utils.get_band_id(raster_file)
    solar_angle_file, sensor_angle_file = \
        utils.get_pixel_angle_files(angle_file, band_id, tmpdir)
    with rasterio.open(solar_angle_file) as src:
        solar_angle_array = src.read(window=TEST_WINDOW)
    with rasterio.open(sensor_angle_file) as src:
        sensor_angle_array = src.read(window=TEST_WINDOW)
    return np.concatenate([solar_angle_array, sensor_angle_array])


@pytest.fixture
def input_srem_worker(cropped_data, cropped_angle):
    data = np.concatenate([cropped_data, cropped_angle])
    input = {
        'data': data,
        'window': TEST_WINDOW,
        'ij': 0,
        'global_args': {
            'wavelength': OLI_WAVELENGTHS[1],
            'dtype': 'int16',
        }
    }
    return input
